package utils

import (
	"net/http"
	"os"

	"github.com/go-resty/resty/v2"
)

// https://github.com/FlareSolverr/FlareSolverr#-requestget
type request struct {
	Cmd        string `json:"cmd"`
	Url        string `json:"url"`
	MaxTimeout int    `json:"maxTimeout"`
}

type solution struct {
	Status   int               `json:"status"`
	Response []byte            `json:"response"`
	Headers  map[string]string `json:"headers"`
}

type response struct {
	Solution solution `json:"solution"`
}

func GET(url string) ([]byte, int, http.Header, error) {
	var (
		client *resty.Client   = resty.New()
		res    *resty.Response = nil
		frurl  string          = ""
		err    error
	)

	if frurl = os.Getenv("FLARERESOLVER"); frurl == "" {
		if res, err := client.R().Get(url); err != nil {
			return nil, 0, nil, err
		} else {
			return res.Body(), res.StatusCode(), res.Header(), nil
		}
	}

	client.R().SetHeader("Content-Type", "application/json")
	client.R().SetBody(request{
		Cmd:        "request.get",
		Url:        url,
		MaxTimeout: 60000,
	})
	client.R().SetResult(&response{})

	if res, err = client.R().Post(url); err != nil {
		return nil, 0, nil, err
	}

	response := res.Result().(*response)
	headers := http.Header{}

	for k, v := range response.Solution.Headers {
		headers.Add(k, v)
	}

	return response.Solution.Response, response.Solution.Status, headers, nil
}
